/**
 * Set custom URLs with titles for user download entries
 *
 * @module package/quiqqer/purchasing/bin/js/backend/controls/CustomUrls
 * @author www.pcsg.de (Patrick Müller)
 *
 * @event onChange [value, this]
 * @event onLoaded [this] - Fires if all statusses have been loaded
 */
define('package/quiqqer/user-downloads/bin/js/backend/controls/CustomUrls', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'qui/controls/buttons/Button',

    'qui/utils/Form',

    'Locale',
    'Mustache',

    'text!package/quiqqer/user-downloads/bin/js/backend/controls/CustomUrls.html',
    'text!package/quiqqer/user-downloads/bin/js/backend/controls/CustomUrls.Entry.html',
    'css!package/quiqqer/user-downloads/bin/js/backend/controls/CustomUrls.css'

], function(QUI, QUIControl, QUILoader, QUIButton, QUIFormUtils, QUILocale, Mustache, template, templateEntry) {
    'use strict';

    const lg = 'quiqqer/user-downloads';

    return new Class({
        Extends: QUIControl,
        Type: 'package/quiqqer/purchasing/bin/js/backend/controls/CustomUrls',

        Binds: [
            '$onImport',
            '$setValue',
            '$onInject',
            'setValue'
        ],

        options: {
            showIcons: false,
            placeholderText: false,
            placeholderSelectable: false
        },

        initialize: function(options) {
            this.parent(options);

            this.$Elm = null;
            this.Loader = new QUILoader();

            this.$fireEvents = true;

            this.addEvents({
                onImport: this.$onImport
            });
        },

        /**
         * event: on import
         */
        $onImport: function() {
            const self = this;

            this.$Input = this.$Elm;
            this.$Input.type = 'hidden';

            this.$Elm = new Element('div', {
                'class': 'quiqqer-user-downloads-customurls',
                html: Mustache.render(template, {})
            }).inject(this.$Input, 'before');

            const ContainerEntries = this.$Elm.getElement('.quiqqer-user-downloads-customurls-entries');
            const ContainerActions = this.$Elm.getElement('.quiqqer-user-downloads-customurls-actions');

            new QUIButton({
                textimage: 'fa fa-plus',
                title: QUILocale.get(lg, 'controls.CustomUrls.btn.add'),
                text: QUILocale.get(lg, 'controls.CustomUrls.btn.add'),
                events: {
                    onClick: () => {
                        this.$getEntryElm().then((Elm) => {
                            Elm.inject(ContainerEntries);
                        });
                    }
                },
                styles: {
                    float: 'right'
                }
            }).inject(ContainerActions);

            this.Loader.inject(this.$Elm);

            if (this.$Input.value !== '') {
                this.Loader.show();

                const entries = JSON.decode(this.$Input.value);
                const promises = [];

                for (let i = 0, len = entries.length; i < len; i++) {
                    const Entry = entries[i];

                    promises.push(this.$getEntryElm({
                        url_title: JSON.encode(Entry.title),
                        url_link: Entry.url
                    }));
                }

                Promise.all(promises).then((result) => {
                    result.forEach((Elm) => {
                        Elm.inject(ContainerEntries);
                    });

                    this.Loader.hide();
                });
            }
        },

        /**
         * Get entry element with optional data filled in.
         *
         * @param {Object} [Data]
         * @return {Promise}
         */
        $getEntryElm: function(Data) {
            const Elm = new Element('table', {
                'class': 'data-table data-table-flexbox quiqqer-user-downloads-customurls-entry',
                html: Mustache.render(templateEntry, {
                    labelTitle: QUILocale.get('quiqqer/quiqqer', 'title'),
                    labelUrl: QUILocale.get(lg, 'controls.CustomUrls.tpl.labelUrl'),
                    descUrl: QUILocale.get(lg, 'controls.CustomUrls.tpl.descUrl')
                })
            });

            if (Data) {
                QUIFormUtils.setDataToNode(Data, Elm);
            }

            new QUIButton({
                icon: 'fa fa-trash',
                title: QUILocale.get(lg, 'controls.CustomUrls.btn.delete'),
                events: {
                    onClick: function() {
                        Elm.destroy();
                    }
                }
            }).inject(Elm.getElement('.quiqqer-user-downloads-customurls-entry-actions'));

            return new Promise((resolve) => {
                QUI.parse(Elm).then(() => {
                    Elm.getElement('input[name="url_link"]').addEvent('change', this.$setValue);

                    const TitleInputControl = QUI.Controls.getById(
                        Elm.getElement('input[name="url_title"]').get('data-quiid')
                    );

                    if (TitleInputControl.isLoaded()) {
                        TitleInputControl.getElm().getElements('input').addEvent('change', this.$setValue);
                        resolve(Elm);

                        return;
                    }

                    TitleInputControl.addEvent('onLoad', () => {
                        TitleInputControl.getElm().getElements('input').addEvent('change', this.$setValue);
                        resolve(Elm);
                    });
                });
            });
        },

        /**
         * Set current value to input
         *
         * @return {Array}
         */
        $setValue: function() {
            const entries = [];
            const entryElements = this.$Elm.getElements('.quiqqer-user-downloads-customurls-entry');

            for (let i = 0, len = entryElements.length; i < len; i++) {
                const EntryElm = entryElements[i];

                const TitleInputControl = QUI.Controls.getById(
                    EntryElm.getElement('input[name="url_title"]').get('data-quiid')
                );

                entries.push({
                    url: EntryElm.getElement('input[name="url_link"]').value.trim(),
                    title: JSON.decode(TitleInputControl.getValue())
                });
            }

            this.$Input.value = JSON.encode(entries);
        }
    });
});