/**
 * Manage payments restrictions for users and groups
 *
 * @module package/quiqqer/user-downloads/bin/js/backend/controls/users/UserDownloads
 * @author www.pcsg.de (Patrick Müller)
 */
define('package/quiqqer/user-downloads/bin/js/backend/controls/users/UserDownloads', [

    'qui/QUI',
    'qui/controls/Control',
    'qui/controls/loader/Loader',
    'qui/controls/windows/Popup',
    'qui/controls/windows/Confirm',
    'qui/controls/buttons/Button',
    'qui/controls/buttons/Separator',

    'controls/groups/Select',
    'controls/grid/Grid',
    'utils/Controls',
    'qui/utils/Form',

    'package/quiqqer/user-downloads/bin/js/backend/UserDownloads',

    'Locale',
    'Mustache',

    'text!package/quiqqer/user-downloads/bin/js/backend/controls/users/UserDownloads.Entry.html',
    'css!package/quiqqer/user-downloads/bin/js/backend/controls/users/UserDownloads.css'

], function(QUI, QUIControl, QUILoader, QUIPopup, QUIConfirm, QUIButton, QUISeparator,
    GroupSelect, Grid, QUIControlUtils, QUIFormUtils, Handler,
    QUILocale, Mustache, templateEntry
) {
    'use strict';

    const lg = 'quiqqer/user-downloads';

    return new Class({

        Extends: QUIControl,
        Type: 'package/quiqqer/user-downloads/bin/js/backend/controls/users/UserDownloads',

        Binds: [
            '$onCreate',
            '$onResize',
            '$listRefresh',
            '$onRefresh',
            '$load',
            '$setGridData',
            '$createRestriction',
            '$toggleActiveStatus',
            '$managePackages',
            '$deleteRestrictions',
            '$onClickEditEntry',
            '$onClickDeleteEntry',
            '$onClickCreateEntry'
        ],

        options: {
            userId: false       // QUIQQER user id
        },

        initialize: function(options) {
            this.parent(options);

            this.Loader = new QUILoader();
            this.$Grid = null;
            this.$GridParent = null;

            this.addEvents({
                onInject: this.$onInject,
                onRefresh: this.$onRefresh,
                onResize: this.$onResize
            });
        },

        /**
         * Event: onImport
         */
        $onInject: function() {
            this.$Elm.addClass(
                'quiqqer-user-downloads'
            );

            this.Loader.inject(this.$Elm);
            this.$load();
        },

        /**
         * event: onResize
         */
        $onResize: function() {
            if (this.$GridParent && this.$Grid) {
                const size = this.$GridParent.getSize();

                this.$Grid.setHeight(size.y);
                this.$Grid.resize();
            }
        },

        /**
         * Load memberships management
         */
        $load: function() {
            const self = this;

            this.$Elm.set(
                'html',
                '<div class="quiqqer-user-downloads-table"></div>'
            );

            const Content = this.$Elm;

            this.$GridParent = Content.getElement(
                '.quiqqer-user-downloads-table'
            );

            const GridAttributes = {
                columnModel: [
                    {
                        header: QUILocale.get(lg, 'controls.UserDownloads.tbl.header.title'),
                        dataIndex: 'titleDisplay',
                        dataType: 'string',
                        width: 250
                    }, {
                        header: QUILocale.get(lg, 'controls.UserDownloads.tbl.header.description'),
                        dataIndex: 'descriptionDisplay',
                        dataType: 'string',
                        width: 250
                    }, {
                        header: QUILocale.get(lg, 'controls.UserDownloads.tbl.header.fileCount'),
                        dataIndex: 'fileCount',
                        dataType: 'int',
                        width: 250
                    }, {
                        dataIndex: 'id',
                        dataType: 'number',
                        hidden: true
                    }
                ],
                pagination: false,
                serverSort: false,
                selectable: true,
                multipleSelection: true
            };

            GridAttributes.buttons = [
                {
                    name: 'add',
                    text: QUILocale.get(lg, 'controls.UserDownloads.tbl.btn.add'),
                    textimage: 'fa fa-plus',
                    events: {
                        onClick: this.$onClickCreateEntry
                    }
                }, {
                    name: 'edit',
                    text: QUILocale.get(lg, 'controls.UserDownloads.tbl.btn.edit'),
                    textimage: 'fa fa-edit',
                    events: {
                        onClick: this.$onClickEditEntry
                    }
                }, {
                    name: 'delete',
                    text: QUILocale.get(lg, 'controls.UserDownloads.tbl.btn.delete'),
                    textimage: 'fa fa-trash',
                    events: {
                        onClick: this.$onClickDeleteEntry
                    }
                }
            ];

            this.$Grid = new Grid(this.$GridParent, GridAttributes);

            this.$Grid.addEvents({
                onDblClick: self.$onClickEditEntry,
                onClick: function() {
                    const selected = self.$Grid.getSelectedData().length;
                    const TableButtons = self.$Grid.getAttribute('buttons');

                    if (!Object.getLength(TableButtons)) {
                        return;
                    }

                    TableButtons.delete.enable();

                    if (selected === 1) {
                        TableButtons.edit.enable();
                    } else {
                        TableButtons.edit.disable();
                    }
                },
                onRefresh: this.$listRefresh
            });

            this.resize();
            this.$Grid.refresh();
        },

        /**
         * Event: onRefresh
         */
        $onRefresh: function() {
            if (this.$Grid) {
                this.$Grid.refresh();
            }
        },

        /**
         * Refresh membership list
         *
         * @param {Object} Grid
         */
        $listRefresh: function(Grid) {
            const self = this;

            if (!this.$Grid) {
                return;
            }

            const TableButtons = self.$Grid.getAttribute('buttons');

            TableButtons.edit.disable();
            TableButtons.delete.disable();

            this.Loader.show();

            Handler.getList(this.getAttribute('userId')).then(function(GridData) {
                self.Loader.hide();

                const currentLang = QUILocale.getCurrent();

                for (let i = 0, len = GridData.data.length; i < len; i++) {
                    const Row = GridData.data[i];

                    Row.titleDisplay = Row.title[currentLang];
                    Row.descriptionDisplay = Row.description[currentLang];
                    Row.fileCount = Row.download_items.quiqqerFiles.length +
                        Row.download_items.urls.length;
                }

                self.$Grid.setData(GridData);
            });
        },

        /**
         * Create new download entry
         */
        $onClickCreateEntry: function() {
            const self = this;

            let Form;
            let CustomUrlsControl;

            // open popup
            new QUIConfirm({
                maxHeight: 800,
                maxWidth: 1200,
                autoclose: false,

                title: QUILocale.get(lg, 'controls.UserDownloads.create.title'),
                icon: 'fa fa-plus',

                cancel_button: {
                    text: false,
                    textimage: 'icon-remove fa fa-remove'
                },
                ok_button: {
                    text: false,
                    textimage: 'icon-ok fa fa-check'
                },

                events: {
                    onOpen: function(Win) {
                        Win.setContent(Mustache.render(templateEntry, {
                            labelTitle: QUILocale.get('quiqqer/quiqqer', 'title'),
                            labelDescription: QUILocale.get('quiqqer/quiqqer', 'description'),
                            headerFiles: QUILocale.get(lg, 'controls.UserDownloads.Entry.tpl.headerFiles'),
                            headerUrls: QUILocale.get(lg, 'controls.UserDownloads.Entry.tpl.headerUrls')
                        }));

                        Form = Win.getContent().getElement('form');

                        Win.Loader.show();

                        const Content = Win.getContent();

                        QUI.parse(Content).then(function() {
                            CustomUrlsControl = QUI.Controls.getById(
                                Content.getElement('input[name="urls"').get('data-quiid')
                            );

                            Win.Loader.hide();
                        });
                    },
                    onSubmit: function(Win) {
                        let Data = QUIFormUtils.getFormData(Form);

                        Win.Loader.show();

                        Handler.create(self.getAttribute('userId'), Data).then(function() {
                            self.$Grid.refresh();
                            Win.close();
                        }).catch(function() {
                            Win.Loader.hide();
                        });
                    }
                }
            }).open();
        },

        /**
         * Edit download entry
         */
        $onClickEditEntry: function() {
            const self = this;
            const Row = this.$Grid.getSelectedData()[0];

            let Form;
            let CustomUrlsControl;

            // open popup
            new QUIConfirm({
                maxHeight: 800,
                maxWidth: 1200,
                autoclose: false,

                title: QUILocale.get(lg, 'controls.UserDownloads.edit.title'),
                icon: 'fa fa-edit',

                cancel_button: {
                    text: false,
                    textimage: 'icon-remove fa fa-remove'
                },
                ok_button: {
                    text: false,
                    textimage: 'icon-ok fa fa-check'
                },

                events: {
                    onOpen: function(Win) {
                        Win.setContent(Mustache.render(templateEntry, {
                            labelTitle: QUILocale.get('quiqqer/quiqqer', 'title'),
                            labelDescription: QUILocale.get('quiqqer/quiqqer', 'description'),
                            headerFiles: QUILocale.get(lg, 'controls.UserDownloads.Entry.tpl.headerFiles'),
                            headerUrls: QUILocale.get(lg, 'controls.UserDownloads.Entry.tpl.headerUrls')
                        }));

                        Form = Win.getContent().getElement('form');

                        Win.Loader.show();

                        // Parse data
                        const FormData = {
                            title: JSON.encode(Row.title),
                            description: JSON.encode(Row.description),
                            files: Row.download_items.quiqqerFiles.join(','),
                            urls: JSON.encode(Row.download_items.urls)
                        };

                        QUIFormUtils.setDataToForm(FormData, Form);

                        const Content = Win.getContent();

                        QUI.parse(Content).then(function() {
                            CustomUrlsControl = QUI.Controls.getById(
                                Content.getElement('input[name="urls"').get('data-quiid')
                            );

                            Win.Loader.hide();
                        });
                    },
                    onSubmit: function(Win) {
                        let Data = QUIFormUtils.getFormData(Form);

                        Win.Loader.show();

                        Handler.update(Row.id, Data).then(function() {
                            self.$Grid.refresh();
                            Win.close();
                        }).catch(function() {
                            Win.Loader.hide();
                        });
                    }
                }
            }).open();
        },

        /**
         * Delete download entries
         */
        $onClickDeleteEntry: function() {
            const selected = this.$Grid.getSelectedData();
            const ids = [];
            const titles = [];

            for (let i = 0, len = selected.length; i < len; i++) {
                const Row = selected[i];

                ids.push(Row.id);
                titles.push('<li>' + Row.titleDisplay + ' (#' + Row.id + ')' + '</li>');
            }

            new QUIConfirm({
                maxHeight: 500,
                maxWidth: 650,

                autoclose: false,
                backgroundClosable: true,

                information: QUILocale.get(lg, 'controls.UserDownloads.delete.information', {
                    entries: titles.join('')
                }),
                title: QUILocale.get(lg, 'controls.UserDownloads.delete.title'),
                texticon: 'fa fa-trash',
                text: QUILocale.get(lg, 'controls.UserDownloads.delete.text'),
                icon: 'fa fa-trash',

                cancel_button: {
                    text: false,
                    textimage: 'icon-remove fa fa-remove'
                },
                ok_button: {
                    text: QUILocale.get(lg, 'controls.UserDownloads.delete.btn.submit'),
                    textimage: 'icon-ok fa fa-check'
                },
                events: {
                    onSubmit: (Win) => {
                        Win.Loader.show();

                        Handler.delete(ids).then(() => {
                            this.$Grid.refresh();
                            Win.close();
                        }).catch(() => {
                            Win.Loader.hide();
                        });
                    }
                }
            }).open();
        }
    });
});
